@@ -1,4 +1,5 @@
 import numpy as np
+
 from pylops import LinearOperator
 from pylops.utils.backend import get_array_module
 
@@ -29,18 +30,18 @@ class Conj(LinearOperator):
 
     .. math::
 
-        y_{i} = \Re\{x_{i}\} - i\Im\{x_{i}\} \quad \forall i=0,...,N
+        y_{i} = \Re\{x_{i}\} - i\Im\{x_{i}\} \quad \forall i=0,\ldots,N-1
 
     In adjoint mode:
 
     .. math::
 
-        x_{i} = \Re\{y_{i}\} - i\Im\{y_{i}\} \quad \forall i=0,...,N
+        x_{i} = \Re\{y_{i}\} - i\Im\{y_{i}\} \quad \forall i=0,\ldots,N-1
 
     """
-    def __init__(self, dims, dtype='complex128'):
-        self.shape = (np.prod(np.array(dims)),
-                      np.prod(np.array(dims)))
+
+    def __init__(self, dims, dtype="complex128"):
+        self.shape = (np.prod(np.array(dims)), np.prod(np.array(dims)))
         self.dtype = np.dtype(dtype)
         self.explicit = False
         self.clinear = False
