@@ -1,17 +1,18 @@
 import logging
 
 import numpy as np
+
 from pylops import LinearOperator
 from pylops.utils.backend import get_array_module
 
-logging.basicConfig(format='%(levelname)s: %(message)s', level=logging.WARNING)
+logging.basicConfig(format="%(levelname)s: %(message)s", level=logging.WARNING)
 
 
 class Regression(LinearOperator):
     r"""Polynomial regression.
 
     Creates an operator that applies polynomial regression to a set of points.
-    Values along the t-axis must be provided while initializing the operator.
+    Values along the :math:`t`-axis must be provided while initializing the operator.
     The coefficients of the polynomial regression form the model vector to
     be provided in forward mode, while the values of the regression
     curve shall be provided in adjoint mode.
@@ -19,7 +20,7 @@ class Regression(LinearOperator):
     Parameters
     ----------
     taxis : :obj:`numpy.ndarray`
-        Elements along the t-axis.
+        Elements along the :math:`t`-axis.
     order : :obj:`int`
         Order of the regressed polynomial.
     dtype : :obj:`str`, optional
@@ -36,7 +37,7 @@ class Regression(LinearOperator):
     Raises
     ------
     TypeError
-        If ``t`` is not :obj:`numpy.ndarray`.
+        If ``taxis`` is not :obj:`numpy.ndarray`.
 
     See Also
     --------
@@ -47,9 +48,9 @@ class Regression(LinearOperator):
     The Regression operator solves the following problem:
 
     .. math::
-        y_i = \sum_{n=0}^{order} x_n t_i^n  \qquad \forall i=1,2,...,N
+        y_i = \sum_{n=0}^\text{order} x_n t_i^n  \qquad \forall i=0,1,\ldots,N-1
 
-    where :math:`N` represents the order of the chosen polynomial. We can
+    where :math:`N` represents the number of points in ``taxis``. We can
     express this problem in a matrix form
 
     .. math::
@@ -58,45 +59,45 @@ class Regression(LinearOperator):
     where
 
     .. math::
-        \mathbf{y}= [y_1, y_2,...,y_N]^T,
-        \qquad \mathbf{x}= [x_0, x_1,...,x_{order}]^T
+        \mathbf{y}= [y_0, y_1,\ldots,y_{N-1}]^T,
+        \qquad \mathbf{x}= [x_0, x_1,\ldots,x_\text{order}]^T
 
     and
 
     .. math::
         \mathbf{A}
         = \begin{bmatrix}
-            1  & t_{1} & t_{1}^2 & .. & t_{1}^{order}  \\
-            1  & t_{2} & t_{2}^2 & .. & t_{1}^{order}  \\
-            .. & ..    & ..      & .. & ..             \\
-            1  & t_{N} & t_{N}^2 & .. & t_{N}^{order}  \\
-        \end{bmatrix}
+            1      & t_{0}  & t_{0}^2 & \ldots & t_{0}^\text{order}  \\
+            1      & t_{1}  & t_{1}^2 & \ldots & t_{1}^\text{order}  \\
+            \vdots & \vdots & \vdots  & \ddots & \vdots             \\
+            1      & t_{N-1}  & t_{N-1}^2 & \ldots & t_{N-1}^\text{order}
+        \end{bmatrix}_{N\times \text{order}+1}
 
     """
-    def __init__(self, taxis, order, dtype='float64'):
+
+    def __init__(self, taxis, order, dtype="float64"):
         ncp = get_array_module(taxis)
         if not isinstance(taxis, ncp.ndarray):
-            logging.error('t must be numpy.ndarray...')
-            raise TypeError('t must be numpy.ndarray...')
+            logging.error("t must be numpy.ndarray...")
+            raise TypeError("t must be numpy.ndarray...")
         else:
             self.taxis = taxis
         self.order = order
-        self.shape = (len(self.taxis), self.order+1)
+        self.shape = (len(self.taxis), self.order + 1)
         self.dtype = np.dtype(dtype)
         self.explicit = False
 
     def _matvec(self, x):
         ncp = get_array_module(x)
         y = ncp.zeros_like(self.taxis)
-        for i in range(self.order+1):
-            y += x[i]*self.taxis**i
+        for i in range(self.order + 1):
+            y += x[i] * self.taxis ** i
         return y
 
     def _rmatvec(self, x):
         ncp = get_array_module(x)
 
-        return ncp.vstack([ncp.dot(self.taxis**i, x)
-                           for i in range(self.order+1)])
+        return ncp.vstack([ncp.dot(self.taxis ** i, x) for i in range(self.order + 1)])
 
     def apply(self, t, x):
         """Return values along y-axis given certain ``t`` location(s) along
